#include "Gadget/Engine.h"
#include "Particle.h"

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	Core G;
	G.Window.SetSize(1024, 768);
	G.Window.Open();
	glDisable(GL_DEPTH_TEST); //Ignore depth

	SoundEngine S; //Only create one instance of the SoundEngine class

	//Load a sound
	Sound shotSnd;
	shotSnd.Load("Sounds/pulseShot.wav", &S, false, false);

	//Load the ship graphic
	Image shipImg;
	shipImg.Load("Images/ship1.jpg");

	//Create a sprite and set its size, position, rotation
	Sprite shipSpr(100.0f, 100.0f);
	shipSpr.rotation = 90.0f;
	shipSpr.x = 512.0f;
	shipSpr.y = 300.0f;

	//We will use a timer to limit the firing rate
	float shotTimer = 0.0f;

	//Initialize the Particle class
	Particle::Init(&G);
	//Create an empty list
	list<Particle> particles;

	while(G.Window.IsOpen())//loop as long as the window is open
	{
		G.Renderer.BeginFrame(); G.Renderer.Render2D(); //render a 2D scene

			//ESC closes the window
			if(G.Keyboard.GetKeyHold(GLFW_KEY_ESC)) {G.Window.Close();}	

			//Create new particles
			shotTimer += G.Timer.GetElapsedTime();//Track the time since the last shot
			//Shoot if 0.2 seconds have passed and the spacebar is pressed or button 2 on a controller 0 is pressed
			if(shotTimer>0.2f && (G.Keyboard.GetKeyHold(GLFW_KEY_SPACE) || G.Joystick.GetButtonHold(0,2)))
			{
				Particle tempPart;
				tempPart.position.x = shipSpr.x;
				tempPart.position.y = shipSpr.y;
				//Make the particle face the same direction as the ship
				tempPart.spr.rotation = shipSpr.rotation;
				//make the particle move in the direction that the ship is facing
				tempPart.velocity = tempPart.velocity.AngleToVector(shipSpr.rotation)*600.0f;
				particles.push_back(tempPart);

				shotSnd.Play();//Play the shooting sound

				shotTimer = 0.0f;//reset the timer
			}

			//Loop through the list of particles and call the step function
			for(list<Particle>::iterator i = particles.begin(); i != particles.end();)
			{
				if(i->timer>1.0f)//erase particles older than 1 second
				{
					i = particles.erase(i);
				}
				else
				{
					//Its not time to erase this one so we will call its Strp function
					i->Step(); 
					i++; //Get the next Particle in the list
				}
			}

			//Move the ship using the keyboard or joystick(controller)
			//Axis 0 on a controller is left and right. The value ranges from -1 to 1
			if(G.Keyboard.GetKeyHold(GLFW_KEY_LEFT) || G.Joystick.GetAxis(0, 0)<-0.5f)
			{
				shipSpr.rotation += G.Timer.GetElapsedTime()*200.0f;
			}
			if(G.Keyboard.GetKeyHold(GLFW_KEY_RIGHT) || G.Joystick.GetAxis(0, 0)>0.5f)
			{
				shipSpr.rotation -= G.Timer.GetElapsedTime()*200.0f;
			}
			//Axis 1 on a controller is down and up. My controller just has a directional 
			//pad, so I don't check for values in between.
			if(G.Keyboard.GetKeyHold(GLFW_KEY_UP) || G.Joystick.GetAxis(0, 1)>0.5f)
			{
				Vector2D move;
				//The AngleToVector function returns a vector with a length of 1 and it
				//points in the direction you specify (in degrees).
				move = move.AngleToVector(shipSpr.rotation);
				//The vector's length is then adjusted so that the ship moves 200 pixels/second
				move = move*G.Timer.GetElapsedTime()*200.0f;
				//Move the sprite along the move vector
				shipSpr.x += move.x;
				shipSpr.y += move.y;
			}
			if(G.Keyboard.GetKeyHold(GLFW_KEY_DOWN) || G.Joystick.GetAxis(0, 1)<-0.5f)
			{
				Vector2D move;
				move = move.AngleToVector(shipSpr.rotation);
				move = move*G.Timer.GetElapsedTime()*200.0f;
				shipSpr.x -= move.x;
				shipSpr.y -= move.y;
			}

			//Render the ship
			Shader::Blend(3);
			shipImg.Activate();
			shipSpr.Render();

			Shader::Deactivate();//Always a good idea to turn off any shaders when finished rendering

			S.Update(G.Timer.GetElapsedTime());//The sound engine must be updated constantly
			
		G.Renderer.EndFrame();//finished rendering
	}
	return 0;
}			